from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from ast import literal_eval
from future.utils import iteritems
from past.builtins import basestring
import copy
import logging
import numpy as np
import os
import os.path as osp
import yaml

from utils.collections import AttrDict

logger = logging.getLogger(__name__)

__C = AttrDict()
# Consumers can get config by:
#   from detectron.core.config import cfg
cfg = __C

__C.NAME = 'demo'

__C.RND_SEED = 7

__C.SIAMRPN = AttrDict()
__C.SIAMRPN.BACKBONE = 'alexnet'
__C.SIAMRPN.STRIDE = 8
__C.SIAMRPN.OUTPUT_CONV_CHANNELS = 256
__C.SIAMRPN.ASPECT_RATIO_RANGE = [.2, .33, 0.5, 0.75, 1.0, 1/.75, 2.0, 3., 5.]
__C.SIAMRPN.CLS_SCALE_FACTOR = 1e-3
__C.SIAMRPN.BBOX_SCALE_FACTOR = 1e-3
__C.SIAMRPN.SHARE_WEIGHTS = True
__C.SIAMRPN.FREEZE_CONV_NUM = 0
# __C.SIAMRPN.Z_IMAGE_SIZE = 127
# __C.SIAMRPN.Z_RESPONSE_SIZE = 6
__C.SIAMRPN.Z_CONTEXT_AMOUNT = 0.5
# __C.SIAMRPN.X_IMAGE_SIZE = 255
# __C.SIAMRPN.X_RESPONSE_SIZE = 22
__C.SIAMRPN.WEIGHTS_FILE = ''
__C.SIAMRPN.ENABLE_FACTORIZATION_ARCH = False
__C.SIAMRPN.AR_ATTENTION = 'none'
__C.SIAMRPN.AR_EMBEDDING_INIT = 'zero'
__C.SIAMRPN.RPN_HEAD_LOSS = 'softmax'
__C.SIAMRPN.ANCHOR_SCALES = [8.]
__C.SIAMRPN.ANCHOR_RATIOS = [0.33, 0.5, 1.0, 2.0, 3.0]
__C.SIAMRPN.EMBEDDING_TENSOR_NAME = 'conv5'

__C.RCNN = AttrDict()
__C.RCNN.FEAT_LIST = ['conv4', 'conv2']
__C.RCNN.FEAT_STRIDE_LIST = [8.]
__C.RCNN.MULTI_LAYER_FUSION_TYPE = 'concat'
__C.RCNN.POOL_SIZE = [6, 6]
__C.RCNN.POOL_CHANNEL = 640
__C.RCNN.FEAT_CHANNELS = [384, 256]
__C.RCNN.SIAM_FEAT_FUSION_TYPE = 'concat'
__C.RCNN.HIDDEN_LAYER_CHANNEL = 256
__C.RCNN.NUM_LINEAR_LAYERS = 2
__C.RCNN.HIDDEN_LAYER_FORCE_LINEAR = False
__C.RCNN.BBOX_VAR_INV = [5.0, 5.0, 10.0, 10.0]
__C.RCNN.WEIGHTS_FILE = ''
__C.RCNN.NUM_CLASSES = 2
__C.RCNN.ROI_FEAT_BN= True
__C.RCNN.POOL_CONVS= [256]

# ---------------------------------------------------------------------------- #
# Tracking inference options
# ---------------------------------------------------------------------------- #

__C.INFERENCE = AttrDict()
__C.INFERENCE.WINDOWS = 'cosine'
__C.INFERENCE.PENALTY_K = 0.055
__C.INFERENCE.WINDOWS_INFLUENCE = 0.40
__C.INFERENCE.VERIFICATION_INFLUENCE = 0.50
__C.INFERENCE.LR = 0.250
__C.INFERENCE.Z_IMAGE_SIZE = 127
__C.INFERENCE.Z_RESPONSE_SIZE = 6
__C.INFERENCE.X_IMAGE_SIZE = 271
__C.INFERENCE.X_RESPONSE_SIZE = 24
__C.INFERENCE.NMS_THRESHOLD = 0.5
__C.INFERENCE.NUM_CANDIDATES = 9
__C.INFERENCE.NUM_NEAR_CANDIDATES = 1
__C.INFERENCE.SCORE_THRESH_FACTOR = 0.1
__C.INFERENCE.MAX_SCORE_THRESHOLD = 0.2
__C.INFERENCE.ENABLE_VERIFICATION_BBOX_REFINE = True
__C.INFERENCE.ENABLE_INDEPENDENT_BBOX_WEIGHT = False
__C.INFERENCE.VERIFICATION_BBOX_INFLUENCE = 3.0
__C.INFERENCE.DEBUG_VIS_ON = False
__C.INFERENCE.DEBUG_VIS_STEP = 1
__C.INFERENCE.DEBUG_VIS_START_FRAME = 0
__C.INFERENCE.MAX_NUM_TEMPS = 10
__C.INFERENCE.UPDATE_TYPE = 'naive'

# tmp
__C.INFERENCE.PRE_COSINE_WIN = True

__C.INFERENCE.ENABLE_UPDATE = False
__C.INFERENCE.ENABLE_NEGATIVE_UPDATE = False
__C.INFERENCE.NEGATIVE_WEIGHTS = 0.75

__C.INFERENCE.UPDATE_PO_CHECK_STEP = 15
__C.INFERENCE.UPDATE_PO_MIN_STEP = 25
__C.INFERENCE.UPDATE_PO_RUNNING_MEAN_WEIGHT = 0.9
__C.INFERENCE.UPDATE_PO_RPN_SCORE_MIN = 0.8
__C.INFERENCE.UPDATE_PO_SIM_SCORE_MIN = 0.3
__C.INFERENCE.UPDATE_PO_SIM_SCORE_MAX = 0.6
__C.INFERENCE.UPDATE_PO_WIN_SCORE_MIN = 0.8
__C.INFERENCE.UPDATE_PO_SIZE_PENALTY_MAX = 0.4
__C.INFERENCE.UPDATE_PO_AR_PENALTY_MAX = 0.2
__C.INFERENCE.UPDATE_PO_WEIGHT_FACTOR = 0.7



__C.INFERENCE.UPDATE_NAIVE_WIN_SCORE_STEP = 10
__C.INFERENCE.UPDATE_NAIVE_POS_SIMPOS_SCORE_TH = 0.5
__C.INFERENCE.UPDATE_NAIVE_POS_RPN_SCORE_TH = 0.8
__C.INFERENCE.UPDATE_NAIVE_POS_SIMNGE_SCORE_TH = 0.6

__C.INFERENCE.UPDATE_NAIVE_WIN_SCORE_TH = 0.8

__C.INFERENCE.UPDATE_NAIVE_NEG_RPN_SCORE_TH = 0.70
__C.INFERENCE.UPDATE_NAIVE_NEG_SIMNEG_SCORE_TH = 0.75
__C.INFERENCE.UPDATE_NAIVE_NEG_OVERLAP_TH = 0.3 

__C.INFERENCE.MAX_NUM_NEG_TEMPS = 8

__C.INFERENCE.UPDATE_NAIVE_RUNNING_MEAN_WEIGHT = 0.9