import torch
import numpy as np
from . import nms as _backend
from . import cython_nms as _np_backend

def nms_torch(dets, nms_thresh=0.7):

    if dets.is_cuda:
        with torch.cuda.device(dets.device.index):
            keep_inds = _backend.forward(dets, nms_thresh)
    else:
        keep_inds = _backend.forward(dets, nms_thresh)
    
    return keep_inds


def nms_numpy(dets, nms_thresh=0.7):

    assert isinstance(dets, np.ndarray)
    keep_inds =  _np_backend.nms(dets, nms_thresh)

    return keep_inds



def py_cpu_nms(dets, thresh):
    """Pure Python NMS baseline."""
    x1 = dets[:, 0]
    y1 = dets[:, 1]
    x2 = dets[:, 2]
    y2 = dets[:, 3]
    scores = dets[:, 4]

    areas = (x2 - x1) * (y2 - y1)
    # order = scores.argsort()[::-1]
    order = np.arange(len(scores), dtype=np.int32)
    keep = []
    while order.size > 0:
        i = order[0]
        keep.append(i)
        xx1 = np.maximum(x1[i], x1[order[1:]])
        yy1 = np.maximum(y1[i], y1[order[1:]])
        xx2 = np.minimum(x2[i], x2[order[1:]])
        yy2 = np.minimum(y2[i], y2[order[1:]])

        w = np.maximum(0.0, xx2 - xx1)
        h = np.maximum(0.0, yy2 - yy1)
        inter = w * h
        ovr = inter / (areas[i] + areas[order[1:]] - inter)

        inds = np.where(ovr <= thresh)[0]
        order = order[inds + 1]

    return keep