import numpy as np
import torch

def img_to_tensor(imgs):
    if isinstance(imgs, list):
        num_imgs = len(imgs)
        max_height = max([img.shape[0] for img in imgs])
        max_width = max([img.shape[1] for img in imgs])

        out_imgs = np.array((num_imgs, max_height, max_width, 3), np.float32)
        for ix, img in enumerate(imgs):
            out_imgs[i, :img.shape[0], :img.shape[1], :] = img
        
        out_tensor = torch.from_numpy(out_imgs.transpose(0, 3, 1, 2))
    elif isinstance(imgs, np.ndarray):
        out_tensor = torch.from_numpy(imgs.transpose(2, 0, 1)).unsqueeze(0)
    else:
        raise TypeError('Unknown type {}'.format(type(imgs)))

    return out_tensor


def box_to_aspect_ratio(box):
    if isinstance(box, np.ndarray):
        if len(box.shape) == 2:
            invalid_inds = np.where(box[:, 3] == box[:, 1])[0]
            box[invalid_inds, 3] += 1.0
            aspect_ratio = np.abs(box[:, 2] - box[:, 0]) / \
                            np.abs(box[:, 3] - box[:, 1])
            out_tensor = torch.from_numpy(aspect_ratio)
        elif len(box.shape) == 1:
            if box[3] == box[1]:
                box[3] += 1.0
            aspect_ratio = np.zeros((1, ), np.float32)
            aspect_ratio[0] = np.abs(box[2] - box[0]) / np.abs(box[3] - box[1])
            out_tensor = torch.from_numpy(aspect_ratio)
        else:
            print('Unknown shape {}'.format(box.shape))
            raise NotImplementedError
    elif isinstance(box, list):
        assert len(box) == 4
        if box[3] == box[1]:
            box[3] += 1.0
        aspect_ratio = np.zeros((1, ), np.float32)
        aspect_ratio[0] = np.abs(box[2] - box[0]) / np.abs(box[3] - box[1])
        out_tensor = torch.from_numpy(aspect_ratio)
    else:
        raise TypeError('Unknown type {}'.format(type(box)))
    
    return out_tensor