import sys
import cv2
import torch
import numpy as np
import os
import sys

root_dir = os.path.abspath(os.path.join(os.path.dirname(__file__)))
sys.path.insert(0, os.path.join(root_dir, 'lib'))

from core.siamrcnn_tracker import SiamRCNN_Tracker
import vot_benchmark.vot as vot
from vot_benchmark.vot import Rectangle
from core.config import cfg
import utils.boxes as ubox


cfg.SIAMRPN.WEIGHTS_FILE = os.path.join(root_dir, 'siamrpn_params.pt')
cfg.RCNN.WEIGHTS_FILE = os.path.join(root_dir, 'spm_head_params.pt')
device = torch.device('cuda:{}'.format(0))

tracker = SiamRCNN_Tracker(device)

# start to track
handle = vot.VOT("polygon")
Polygon = handle.region()

box_cxcywh = vot.get_axis_aligned_bbox(Polygon)
# convert to xyxy
box_xyxy = ubox.xcycwh_to_xyxy(box_cxcywh)

image_file = handle.frame()
if not image_file:
    sys.exit(0)
    
im = cv2.imread(image_file)  # HxWxC
im_tensor = torch.from_numpy(np.transpose(im, (2, 0, 1))).float().unsqueeze(0)  # 1*C*H*W
im_tensor = im_tensor.to(device)
tracker.init_tracker(im_tensor, box_xyxy)

while True:
    image_file = handle.frame()
    if not image_file:
        break
    im = cv2.imread(image_file)  # HxWxC
    im_tensor = torch.from_numpy(np.transpose(im, (2, 0, 1))).float().unsqueeze(0)  # 1*C*H*W
    im_tensor = im_tensor.to(device)
    box_xyxy = tracker.predict_next_frame(im_tensor, box_xyxy)
    box_xywh = ubox.xyxy_to_xywh(box_xyxy)
    handle.report(Rectangle(box_xywh[0], box_xywh[1], box_xywh[2], box_xywh[3]))
